#include "appController.h"

appController::appController(){
}

void appController::setup(){

	GUI.useTrueTypeFont("arial.ttf", 10);
	LT.useTrueTypeFont("arial.ttf", 10);
	IP.useTrueTypeFont("arial.ttf", 10);
	
	loadSettings();
	LT.setup(320, 240, 1024, 768);
	IP.setup(1024, 768, 1024, 0, HOST, SENDING_PORT, RECEIVING_PORT);
	
	// Setup of Warping Quad incoming video
	ofPoint srcPos0, srcPos1, srcPos2, srcPos3, topLeftPos;
	srcPos0.set(GUI.getF("WARP_IN_SRC:pt0:x"), GUI.getF("WARP_IN_SRC:pt0:y"));
	srcPos1.set(GUI.getF("WARP_IN_SRC:pt1:x"), GUI.getF("WARP_IN_SRC:pt1:y"));
	srcPos2.set(GUI.getF("WARP_IN_SRC:pt2:x"), GUI.getF("WARP_IN_SRC:pt2:y"));
	srcPos3.set(GUI.getF("WARP_IN_SRC:pt3:x"), GUI.getF("WARP_IN_SRC:pt3:y"));
	topLeftPos.set(670, 170);
	LT.setupWarpIn(srcPos0, srcPos1, srcPos2, srcPos3, topLeftPos);
	
	// Beo GUI
	BGUI.addScrollbar("area resolution", 45, 300, 100, 1500);
	BGUI.addScrollbar("line resolution", 45, 330, 100, 1500);
	BGUI.addScrollbar("laser freeze",    45, 360, 1, 30);
	BGUI.addButton("delete areas",       45, 390, 14);
	BGUI.addButton("delete balls",       45, 420, 14);
	BGUI.addCheck("draw areas",          45, 450, 14);
	BGUI.addCheck("visualize areas",     45, 480, 14);

	// background
	background.loadImage("background_gui_02.jpg");

	letDrawGui = true;
	previewWarp = false;
}

void appController::update(){
	trackLaser();
	checkBgui();
	if(LT.checkLaser()){
		IP.setLaserStatus(true);
		IP.relocateLaserPos(LT.getLaserPos());
	}else{
		IP.setLaserStatus(false);
	}
	IP.update();
}

void appController::trackLaser(){
	//our color tracking settings
	float hue			= GUI.getF("HUE_POINT");
	float hueWidth		= GUI.getF("HUE_WIDTH"); 
	float sat			= GUI.getF("SAT_POINT"); 
	float satWidth		= GUI.getF("SAT_WIDTH"); 
	float value			= GUI.getF("VAL_POINT");
	float valueWidth	= GUI.getF("VAL_WIDTH");

	int blobSize 	= GUI.getI("MIN_BLOB_SIZE");

	//lets process the video data to track that laser
	LT.processFrame(hue, hueWidth, sat,  satWidth, value, valueWidth, blobSize); 

	//only update our color tracking indicator if values have changed				
	if( GUI.hasChanged("HUE_POINT") || GUI.hasChanged("HUE_WIDTH") 
	    || GUI.hasChanged("SAT_POINT") || GUI.hasChanged("SAT_WIDTH") 
		|| GUI.hasChanged("VAL_POINT") || GUI.hasChanged("VAL_WIDTH")){
	
		LT.calcColorRange(hue, hueWidth, sat, satWidth, value);	
	}	
}

void appController::checkBgui(){
	if(BGUI.hasChanged("area resolution")){
		IP.setAreaResolution(BGUI.getF("area resolution"));
	}
	if(BGUI.hasChanged("laser freeze")){
		LT.setFreezedLaserTime(BGUI.getF("laser freeze"));
	}
}

void appController::draw(){

	if(letDrawGui){
		background.draw(0,0);
	
		LT.drawInput(670, 170);
		LT.drawWarpIn();
		LT.drawTrack(670, 460);
		if(previewWarp){
			LT.drawWarpGray(670, 460);
		}
//		LT.drawColorRange(20, 430, 120, 44);
		
		IP.draw();
		GUI.drawAll(405, 180, 150, 14.0);
		BGUI.draw();
	}
}

//----------------------------------------------------
void appController::loadSettings(){

	GUI.readFromFile(ofToDataPath("settings.xml"));

	GUI.addSetting("Hue",				"HUE_POINT", 		0.0f, 0.0f, 1.0f, 0.01f);
	GUI.addSetting("Hue Width",			"HUE_WIDTH", 		0.06f, 0.0f, 1.0f, 0.01f);
	GUI.addSetting("Sat",				"SAT_POINT", 		0.62f,  0.0f, 1.0f, 0.01f);
	GUI.addSetting("Sat Width",			"SAT_WIDTH", 		0.94f,  0.0f, 1.0f, 0.01f);
	GUI.addSetting("Value",				"VAL_POINT", 		0.8f, 0.0f, 1.0f, 0.01f);
	GUI.addSetting("Value Width",		"VAL_WIDTH", 		0.62f, 0.0f, 1.0f, 0.01f);
	GUI.addSetting("Blob Size",			"MIN_BLOB_SIZE",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt0 x",	"WARP_IN_SRC:pt0:x",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt0 y",	"WARP_IN_SRC:pt0:y",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt1 x",	"WARP_IN_SRC:pt1:x",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt1 y",	"WARP_IN_SRC:pt1:y",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt2 x",	"WARP_IN_SRC:pt2:x",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt2 y",	"WARP_IN_SRC:pt2:y",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt3 x",	"WARP_IN_SRC:pt3:x",	2, 0, 30, 1);	
	GUI.addSetting("Warp Quad pt3 y",	"WARP_IN_SRC:pt3:y",	2, 0, 30, 1);	
}

//----------------------------------------------------
void appController::saveSettings(){
	ofPoint * warpPoints = LT.getWarpPoints();
	GUI.setI("WARP_IN_SRC:pt0:x", warpPoints[0].x);
	GUI.setI("WARP_IN_SRC:pt0:y", warpPoints[0].y);
	GUI.setI("WARP_IN_SRC:pt1:x", warpPoints[1].x);
	GUI.setI("WARP_IN_SRC:pt1:y", warpPoints[1].y);
	GUI.setI("WARP_IN_SRC:pt2:x", warpPoints[2].x);
	GUI.setI("WARP_IN_SRC:pt2:y", warpPoints[2].y);
	GUI.setI("WARP_IN_SRC:pt3:x", warpPoints[3].x);
	GUI.setI("WARP_IN_SRC:pt3:y", warpPoints[3].y);
	GUI.saveToFile(ofToDataPath("settings.xml"));
}

//----------------------------------------------------
void appController::reloadSettings(){
	//setCommonText("status: reloading settings from xml");
	GUI.reloadSettings();
	//setCommonText("status: settings reloaded from xml");
}

void appController::keyPressed(int key){
	switch(key){
		case OF_KEY_UP:
			GUI.selectPrev();
			break;
		case OF_KEY_DOWN:
			GUI.selectNext();
			break;
		case OF_KEY_RIGHT:
			GUI.increase();
			break;
		case OF_KEY_LEFT:
			GUI.decrease();
			break;
		case 's':
			saveSettings();
			break;
		case 'r':
			reloadSettings();
			break;
		case 'f': // fullscreen
			ofToggleFullscreen();
			break;
		case 'g': // draw gui
			letDrawGui=true;
			break;
		case 'v': // draw final projection
			letDrawGui=false;
			break;
		case 'w':
			if(previewWarp==false){
				previewWarp=true;
			}else{
				previewWarp=false;
			}
			break;
		case 'p':
			LT.videoConfig();
			break;		
	}
	if(key=='a' || key =='z' || key=='m' || key=='d' || key=='x'){
		IP.keyPressed(key);
	}
}

void appController::keyReleased(int key){
}

void appController::mouseMoved(int x, int y ){
	BGUI.mouseMoved(x, y);
}

void appController::mouseDragged(int x, int y, int button){
	LT.mouseDragged(x, y, button);
	BGUI.mouseDragged(x, y, button);
}

void appController::mousePressed(int x, int y, int button){
	LT.mousePressed(x, y, button);
	IP.mousePressed(x, y, button);
	BGUI.mousePressed(x, y, button);
	if(BGUI.getP("delete areas")){
		IP.deleteAreas();
	}
	if(BGUI.getP("delete balls")){
		IP.deleteAllBalls();
	}
	if( BGUI.getC("draw areas") ){
		IP.setCreateArea(true);
	}else{
		IP.setCreateArea(false);
	}
	if( BGUI.getC("visualize areas") ){
		IP.setAreasVisibility(true);
	}else{
		IP.setAreasVisibility(false);
	}
}

void appController::mouseReleased(int x, int y, int button){
	IP.mouseReleased(x, y, button);
	BGUI.mouseReleased(x, y, button);
}