#include "area.h"

area::area(){
	minX = 9999;
	maxX = 0;
	minY = 9999;
	maxY = 0;
	prevLasPos.set(0, 0);
	isFirstFrameOfMoveArea = true;
	movingHandler = false;
}

void area::create(ofxPoint2f _lPos, float _ar){
	setAreaVertex(_lPos);
	setLineVertex(_lPos);
	setAreaTimer = ofGetElapsedTimeMillis();
	setLineTimer = ofGetElapsedTimeMillis();
	
	areaResolution = _ar;
}

void area::build(ofxPoint2f _lPos){
	if((ofGetElapsedTimeMillis() - setAreaTimer) > areaResolution){
		setAreaVertex(_lPos);
		setAreaTimer = ofGetElapsedTimeMillis();
	}
	
	// Set line vertex sequenced.
	if((ofGetElapsedTimeMillis() - setLineTimer) > LINERESOLUTION){
		setLineVertex(_lPos);
		setLineTimer = ofGetElapsedTimeMillis();
	}
	
	drawCreationLine(_lPos);
}

void area::finishBuilding(){

	// Create the final segments that make the shape of the area
	numOfSegments = areaVertexList.size();
	
	segmentList = new segment[numOfSegments];
	
	for(int i=0; i<numOfSegments; i++){
		if(i<numOfSegments-1){
			segmentList[i].setPoints(areaVertexList[i], areaVertexList[i+1]);
		}else{
			segmentList[i].setPoints(areaVertexList[i], areaVertexList[0]);		
		}
	}
	
	// Delete the array used for drawing the lines during the creation
	// of the area
	if(lineVertexList.size() > 0){
		lineVertexList.erase(lineVertexList.begin(), lineVertexList.begin()+lineVertexList.size());
	}
}

void area::setAreaVertex(ofxPoint2f _lPos){

	// We add a new point to the Area's vertex array (type vector).
	areaVertexList.push_back(_lPos);
	// We set the max and min coordinates of the area
	setSurroundingBox(_lPos);
}

void area::setSurroundingBox(ofxPoint2f _lPos){
	if(_lPos.x < minX){
		minX = _lPos.x;
	}
	if(_lPos.x > maxX){
		maxX = _lPos.x;
	}
	if(_lPos.y < minY){
		minY = _lPos.y;
	}
	if(_lPos.y > maxY){
		maxY = _lPos.y;
	}
}

void area::updateSurroundingBox(){
	minX = 9999;
	maxX = 0;
	minY = 9999;
	maxY = 0;
	
	for(int i=0; i<areaVertexList.size(); i++){		
		setSurroundingBox(areaVertexList[i]);
	}
}

void area::setLineVertex(ofxPoint2f _lPos){

	// We add a new point to the Line's vertex array (type vector).
	lineVertexList.push_back(_lPos);	
}

bool area::isLaserOver(ofxPoint2f _lp){
	if(_lp.x < maxX && _lp.x > minX && _lp.y < maxY && _lp.y > minY){
		return true;
	}else{
		return false;
	}
}

segment * area::getSegmentList(){
	return segmentList;
}

void area::moveArea(ofxPoint2f _lp){
	ofxPoint2f laserPosition = _lp;
	
	if(isLaserOver(laserPosition)){
	
		if(isFirstFrameOfMoveArea){
			prevLasPos = laserPosition;
			isFirstFrameOfMoveArea = false;
		}
		ofxPoint2f difLasPos;
		difLasPos = laserPosition - prevLasPos;

		for(int i=0; i<areaVertexList.size(); i++){		
			areaVertexList[i] = areaVertexList[i] + difLasPos;	
		}
		minX = minX + difLasPos.x;
		maxX = maxX + difLasPos.x;
		minY = minY + difLasPos.y;
		maxY = maxY + difLasPos.y;

		prevLasPos.set(laserPosition.x, laserPosition.y);

	}else if(!isFirstFrameOfMoveArea){
		isFirstFrameOfMoveArea = true;	
	}
}

void area::deleteArea(){
	areaVertexList.erase(areaVertexList.begin(), areaVertexList.begin() + areaVertexList.size());	
	delete [] segmentList;
}

void area::drawCreationLine(ofxPoint2f _lPos){
	for(int i=0; i<lineVertexList.size(); i++){
		if(i!=lineVertexList.size()-1){
			ofLine(lineVertexList[i].x, lineVertexList[i].y, lineVertexList[i+1].x, lineVertexList[i+1].y);
		}else{
			ofLine(lineVertexList[i].x, lineVertexList[i].y, _lPos.x, _lPos.y);		
		}
	}
}

void area::draw(){
	// Draw actual area
	ofSetColor(100, 0, 100);
	ofFill();
	ofBeginShape();
	for(int i=0; i<areaVertexList.size(); i++){
		ofVertex(areaVertexList[i].x, areaVertexList[i].y);
	}
	ofEndShape();

	ofEnableSmoothing();
	ofNoFill();
	ofBeginShape();
	for(int i=0; i<areaVertexList.size(); i++){
		ofVertex(areaVertexList[i].x, areaVertexList[i].y);
	}
	ofEndShape();
	ofDisableSmoothing();
	
	// Draw Surrounding box
	ofSetColor(244, 107, 0);
	ofEnableSmoothing();
	ofNoFill();
	ofBeginShape();
	ofVertex(minX, minY);
	ofVertex(maxX, minY);
	ofVertex(maxX, maxY);
	ofVertex(minX, maxY);
	ofVertex(minX, minY);
	ofEndShape();
	ofDisableSmoothing();
	
	// draw handlers
	ofSetColor(244, 107, 0);
	ofFill();
	for(int i=0; i<areaVertexList.size(); i++){
		ofCircle(areaVertexList[i].x, areaVertexList[i].y, 4);
	}
	ofNoFill();
	ofEnableSmoothing();
	for(int i=0; i<areaVertexList.size(); i++){
		ofCircle(areaVertexList[i].x, areaVertexList[i].y, 4);
	}
	ofDisableSmoothing();
}

int area::getMaxX(){
	return maxX;
}

int area::getMinX(){
	return minX;
}

int area::getMaxY(){
	return maxY;
}

int area::getMinY(){
	return minY;
}

int area::getNumAreaVertex(){
	return areaVertexList.size();
}

ofxPoint2f area::getAreaVertex(int _i){
	return areaVertexList[_i];
}

void area::mouseDragged(int x, int y, int button){
	if(movingHandler){
		areaVertexList[handlerSelected].set(x, y);
	}
}

void area::mousePressed(int x, int y, int button){
	for(int i=0; i<areaVertexList.size(); i++){
		if(x > areaVertexList[i].x - 4 &&
		   x < areaVertexList[i].x + 4 &&
		   y > areaVertexList[i].y - 4 &&
		   y < areaVertexList[i].y + 4	){
		   
			handlerSelected = i;
			movingHandler = true;
		}
	}
}

void area::mouseReleased(int x, int y, int button){
	if(movingHandler){
		areaVertexList[handlerSelected].set(x, y);		
		
		numOfSegments = areaVertexList.size();
		for(int i=0; i<numOfSegments; i++){
			if(i<numOfSegments-1){
				segmentList[i].setPoints(areaVertexList[i], areaVertexList[i+1]);
			}else{
				segmentList[i].setPoints(areaVertexList[i], areaVertexList[0]);		
			}
		}

		updateSurroundingBox();		
//		setSurroundingBox(areaVertexList[handlerSelected]);
		movingHandler = false;
	}
}


