#include "ball.h"

ball::ball(){
}

ball::ball(ofxPoint2f bornPosition, area a){
	x = bornPosition.x;
	y = bornPosition.y;
	myArea = a;
}

ball::ball(ofxPoint2f bornPosition, area _a, float _r, 
		   ofxOscSender _sOsc){
	
	x = bornPosition.x;
	y = bornPosition.y;
	myArea = _a;
	r = _r;
	v.set(0, 0);
	cr = cYr = 255;
	cg = cYg = 3;
	cb = cYb = 184;
	cNr = 214;
	cNg = 0;
	cNb = 18;
	senderOsc = _sOsc;
	tailSize = 20;
	
    m = r * 0.1f;
	f = 0.99f;	
	creationFinished = false;
	erasingMe = killMe = false;
	collided = dissapear = soundStoped = false;
	previousPos.set(x, y);
	
	// type of balls
	mass = bounce = false;
	friction = false;
	ballIsMoving = true;
	
	fadeOutAlpha=1;
	extressTime = 0;
}

ball::~ball(){
	if(bufferPts.size()>0){
		bufferPts.clear();
	}
}

void ball::finishCreation(ofxPoint2f _lp, int _id){
	myId = _id;
	setVelocity(_lp);
	ballCollList = new ball[1];
	creationFinished = true;
	if(friction){		
		setSamplePosition();
		setSamplePitch();
		setPan();
		sendOscMessage("friction", "play");
	}
}

void ball::checkCreationConditions(vector <ball> _bList, ofxPoint2f _lp){
	if(!creationFinished){
		vector <ball> bList = _bList;
		ofxPoint2f lp = _lp;
	  
		// Check if the ball is on top of others
		if(bList.size()>1){
			for(int i=0; i<bList.size()-1; i++){
				ball b = bList[i];
				if(distance(b) < r + b.r){
					
					// Set colors
					cr = cNr;
					cg = cNg;
					cb = cNb;
					
					// Kill the ball
					killMe = true;
					break;
				}else{
				
					// Set colors
					cr = cYr;
					cg = cYg;
					cb = cYb;
	
					// Do not kill the ball
					killMe = false;          
				}
			}// end for loop
		}// end if bList.size()>1
		
		setVelocity(lp);
		
		if(mass){
			setRadius(lp);
		}
	}// end if creationFinished
}

void ball::update(){
	if(creationFinished){
		x += v.x;
		y += v.y;
		v.x *= f;
		v.y *= f;
		checkShapeWall();
		setSpeed();
		
		if(speed > 0.01){
			ballIsMoving = true;
		}else{
			ballIsMoving = false;
			if(friction){
				if(bufferPts.size()>0){
					bufferPts.clear();
				}
			}
		}
	
		if(friction){
			addPointToBuffer();
			setSamplePosition();
			setSamplePitch();
			setPan();

			if(ballIsMoving){
				sendOscMessage("friction", "update");
				soundStoped = false;
			}else if(!soundStoped){
				sendOscMessage("friction", "stop");
				soundStoped = true;
			}
		}
		
		// Wave Visualization Update
		if(waveList.size()>0){
			for(int i=0; i<waveList.size(); i++){
				waveList[i].update(x, y);
				if(waveList[i].getKillMe()){
					waveList.erase(waveList.begin()+i);
				}	
			}
		}
		
		// Tremor Visualization Update
		if(tremorList.size()>0){
			for(int i=0; i<tremorList.size(); i++){
				tremorList[i].update(x, y);
				if(tremorList[i].getKillMe()){
					tremorList.erase(tremorList.begin()+i);
				}
			}
		}
	}
	
	if(previousPos.x == x && previousPos.y == y){
		isReallyMoving = false;
	}else{
		isReallyMoving = true;
	}
	
	previousPos.set(x, y);
	
	// Fade Out Killer
	if(dissapear){
		if(fadeOutAlpha>0){
			fadeOutAlpha -= 0.01;
		}else{
			// Here we have to send a message to stop the sound 
			// of the ball that was just deleted
			killSound();
			killMe = true;
		}
	}

}

void ball::setVelocity(ofxPoint2f _lp){
	ofxPoint2f lp = _lp;	//laser position
	
	// Distance no biger than 300
	float dist = MIN(distance(lp), 300);
    float ang = atan2(x-lp.x, y-lp.y);
	float sine = sin(ang);
	float cosine = cos(ang);
	
	// Map velocity from distance between 0 and 10
	float vx = ofMap(dist, 0, 300, 0, 10) * sine;
	float vy = ofMap(dist, 0, 300, 0, 10) * cosine;
    v.set(vx, vy);

	// Set volume and Pan
	setVolume(sine, cosine);
	setPan();	
}

void ball::setVolume(float _sin, float _cos){
	float sine = _sin;
	float cosine = _cos;
	
	vol =  (	(v.x*sine) + (v.y*cosine)	 )/2;
	vol = ofMap(myAbs(vol), 0, 5, 1, 0.1);
	vol = MAX(MIN(vol, 1), 0.1);
}

void ball::setSpeed(){
    float ang = atan2(x, y);
	float sine = sin(ang);
	float cosine = cos(ang);
	
	speed =  (	(myAbs(v.x)*sine) + (myAbs(v.y)*cosine)	 );
}

void ball::setPan(){
	pan = ofMap(x, myArea.getMinX(), myArea.getMaxX(), -1, 1);
}

void ball::setRadius(ofxPoint2f _lp){
    ofxPoint2f lp = _lp;
    // Create radius based on distance between mouse and ball
	// rLimit constrains the radio to the size of the area.
	float distX = (myArea.getMaxX() - myArea.getMinX())/2;
	float distY = (myArea.getMaxY() - myArea.getMinY())/2;
	float rLimit = MIN(distX, distY)-4;
	
    float dist = MIN(distance(lp), 200);
    r = ofMap(dist, 0, 200, rLimit, 5);
    m = r * 0.1f;
}

void ball::setSamplePosition(){
	position = ofMap(x, myArea.getMinX(), myArea.getMaxX(), 0, 1);
}

void ball::setSamplePitch(){
	pitch = ofMap(y, myArea.getMinY(), myArea.getMaxY(), 2.0f, 0.5f);
}

void ball::addPointToBuffer(){
	ofxPoint2f pt;
	pt.set(x, y);
	bufferPts.push_back(pt);
	
	if (bufferPts.size() > tailSize){
		bufferPts.erase(bufferPts.begin());
	}
}

void ball::checkQuadWall(){
	int maxX = myArea.getMaxX();
	int minX = myArea.getMinX();
	int maxY = myArea.getMaxY();
	int minY = myArea.getMinY();
	
	if(x > maxX-r){
				// set sound
				setPan();
				sendOscMessage("collision");
				addTremor();
		x = maxX-r;
		v.x *= -1;
	}else if (x < minX+r){
				// set sound
				setPan();
				sendOscMessage("collision");
				addTremor();
		x = minX+r;
		v.x *= -1;
	}else if(y < minY+r){
				// set sound
				setPan();
				sendOscMessage("collision");
				addTremor();
		y = minY+r;
		v.y *= -1;
	}else if(y > maxY-r){
				// set sound
				setPan();
				sendOscMessage("collision");
				addTremor();
		y = maxY-r;
		v.y *= -1;
	}
}

void ball::checkShapeWall(){
	int segmentListLength = myArea.numOfSegments;
	segment * segmentList = myArea.getSegmentList();
	
	for(int i=0; i<segmentListLength; i++){

		ofxPoint2f delta;
		delta.x = x - segmentList[i].x;
		delta.y = y - segmentList[i].y;
		
		float cosine = cos(segmentList[i].rot);
		float sine = sin(segmentList[i].rot);
		
		float groundXTemp = cosine * delta.x + sine * delta.y;
		float groundYTemp = cosine * delta.y - sine * delta.x;
		float velocityXTemp = cosine * v.x + sine * v.y;
		float velocityYTemp = cosine * v.y - sine * v.x;

		if(segmentList[i].isOnTop()){
			if(groundYTemp < r && 
				x > segmentList[i].pt1.x && 
				x < segmentList[i].pt2.x){
			
				groundYTemp = r;
				velocityYTemp *= -1.0;
		
				// set sound
				setPan();
				sendOscMessage("collision");
				addTremor();
			}		
		}else{
			if(groundYTemp < r && 
				x < segmentList[i].pt1.x && 
				x > segmentList[i].pt2.x){
      
				groundYTemp = r;
				velocityYTemp *= -1.0;

				// set sound
				setPan();
				sendOscMessage("collision");
				addTremor();
			}      
		}
		
		delta.x = cosine * groundXTemp - sine * groundYTemp;
		delta.y = cosine * groundYTemp + sine * groundXTemp;
		v.x = cosine * velocityXTemp - sine * velocityYTemp;
		v.y = cosine * velocityYTemp + sine * velocityXTemp;
		x = segmentList[i].x + delta.x;  
		y = segmentList[i].y + delta.y;    
	}
	
	checkQuadWall();
}

void ball::checkObjectCollision(ball oBall){

	if(creationFinished && oBall.creationFinished){
		ofxVec2f bVect;
		bVect.set(oBall.x - x, oBall.y - y);

		float bVectMag = sqrt(bVect.x * bVect.x + bVect.y * bVect.y);
    
		if (bVectMag < r + oBall.r && !dissapear && !oBall.getDissapeared()){
		
			if(collided){
				dissapear = true;
			}
			if(oBall.getCollided()){
				oBall.setDissapeared(true);
			}

			if(!collided && !dissapear && !oBall.getCollided() && !oBall.getDissapeared()){

				if(ofGetElapsedTimeMillis() - extressTime > 300){
					setPan();
					sendOscMessage("collision");
					oBall.setPan();
					oBall.sendOscMessage("collision");
				
					waveList.push_back(wave());
					waveList[waveList.size()-1].setup(x, y, r, cYr, cYg, cYb);
					oBall.addWave(oBall.x, oBall.y, oBall.r, oBall.getRcolor(), oBall.getGcolor(), oBall.getBcolor());
//					dissapear = true;
					extressTime = ofGetElapsedTimeMillis();
				}
				
				float theta  = atan2(bVect.y, bVect.x);
				float sine = sin(theta);
				float cosine = cos(theta);
		
				ball bTemp0;
				ball bTemp1;
				bTemp1.x  = cosine * bVect.x + sine * bVect.y;
				bTemp1.y  = cosine * bVect.y - sine * bVect.x;
		
				// rotate Temporary velocities
				ofxVec2f vTemp0;
				ofxVec2f vTemp1;
				vTemp0.x  = cosine * v.x + sine * v.y;
				vTemp0.y  = cosine * v.y - sine * v.x;
				vTemp1.x  = cosine * oBall.v.x + sine * oBall.v.y;
				vTemp1.y  = cosine * oBall.v.y - sine * oBall.v.x;
	
				ofxVec2f vFinal0;
				ofxVec2f vFinal1;
				// final rotated velocity for b[0]
				vFinal0.x = ((m - oBall.m) * vTemp0.x + 2 * oBall.m * vTemp1.x) 
							/ (m + oBall.m);
				vFinal0.y = vTemp0.y;
				// final rotated velocity for b[0]
				vFinal1.x = ((oBall.m - m) * vTemp1.x + 2 * m * vTemp0.x) 
							/ (m + oBall.m);
				vFinal1.y = vTemp1.y;
		
				// hack to avoid clumping
				bTemp0.x += vFinal0.x;
				bTemp1.x += vFinal1.x;
		
				// rotate balls
				ball bFinal0;
				ball bFinal1;
				bFinal0.x = cosine * bTemp0.x - sine * bTemp0.y;
				bFinal0.y = cosine * bTemp0.y + sine * bTemp0.x;
				bFinal1.x = cosine * bTemp1.x - sine * bTemp1.y;
				bFinal1.y = cosine * bTemp1.y + sine * bTemp1.x;
	
				// update balls to screen position
				oBall.x = x + bFinal1.x;
				oBall.y = y + bFinal1.y;
				x = x + bFinal0.x;
				y = y + bFinal0.y;
		
				// update velocities
				v.x = cosine * vFinal0.x - sine * vFinal0.y;
				v.y = cosine * vFinal0.y + sine * vFinal0.x;
				oBall.v.x = cosine * vFinal1.x - sine * vFinal1.y;
				oBall.v.y = cosine * vFinal1.y + sine * vFinal1.x;
			}
			
			collided = true;
			oBall.setCollided(true);

		}else if(collided){
			collided = false;
		}else if(oBall.getCollided()){
			oBall.setCollided(false);
		}
	}
	ballCollList[0] = oBall;

}

ball ball::getCollBall(){
	return ballCollList[0];
}

void ball::sendOscMessage(string _type, string _action){
	if(_type == "collision"){
		ofxOscMessage m;
		m.setAddress("/collision");
		m.addStringArg("true");
		m.addIntArg(sample);
		m.addFloatArg(vol+0.4);
		m.addFloatArg(pan);
		senderOsc.sendMessage(m);
	}
	if(_type == "friction"){
		ofxOscMessage w;
		w.setAddress("/friction");
		w.addStringArg(_action);	// 1
		w.addIntArg(myId);			// 2
		w.addIntArg(mySoundId);		// 3
		w.addIntArg(sample);		// 4
		w.addFloatArg(vol);			// 5
		w.addFloatArg(pan);			// 6
		w.addFloatArg(position);	// 7
		w.addFloatArg(pitch);		// 8
		senderOsc.sendMessage(w);
	}
}

void ball::killSound(){
	if(friction){
		sendOscMessage("friction", "stop");
	}
}

void ball::setType(int _cr, int _cg, int _cb, int _s){
	
	//Color
	cr = cYr = _cr;
	cg = cYg = _cg;
	cb = cYb = _cb;
	
	//Sound
	sample = _s;
}

void ball::setErasingMe(bool _e){
	erasingMe = _e;
}

bool ball::getErasingMe(){
	return erasingMe;
}

bool ball::getKillMe(){
	return killMe;
}

bool ball::getDissapeared(){
	return dissapear;
}

void ball::setDissapeared(bool _d){
	dissapear = _d;
}

bool ball::getCollided(){
	return collided;
}

void ball::setCollided(bool _c){
	collided = _c;
}

void ball::setBounce(bool _bounce){
	bounce = _bounce;
}

bool ball::getBounce(){
	return bounce;
}

void ball::setMass(bool _mass){
	mass = _mass;
}

void ball::setFriction(bool _frc){
	friction = _frc;
	if(friction == true){
		f = 0.99f;
	}else{
		f = 1;
	}
}

void ball::setSoundId(int _soundId){
	mySoundId = _soundId;
}

bool ball::getIsMoving(){
	return isReallyMoving;
}

int ball::getRcolor(){
	return cYr;
}

int ball::getGcolor(){
	return cYg;
}

int ball::getBcolor(){
	return cYb;
}

void ball::addWave(float _x, float _y, float _r, int _cr, int _cg, int _cb){
	waveList.push_back(wave());
	waveList[waveList.size()-1].setup(_x, _y, _r, _cr, _cg, _cb);
}

void ball::addTremor(){
	tremorList.push_back(tremor());
	tremorList[tremorList.size()-1].setup(x, y, r, cYr, cYg, cYb);
}

void ball::draw(){
	if(friction){
		drawBuffer();
	}
	if(erasingMe){
		drawErase(x, y, 200);
	}else if(bounce){
		drawBounce(x, y, 155);
	}else{
		drawNeutral(x, y, 155);
	}

	// Wave visuazitation draw
	if(waveList.size()>0){
		for(int i=0; i<waveList.size(); i++){
			waveList[i].draw();
		}
	}		
	// Tremor visuazitation draw
	if(tremorList.size()>0){
		for(int i=0; i<tremorList.size(); i++){
			tremorList[i].draw();
		}
	}	
}

void ball::drawSetup(ofxPoint2f _lp){
	ofxPoint2f lp = _lp;
	
	//Draw Line
	ofEnableSmoothing();
	ofEnableAlphaBlending();
	ofSetColor(cr, cg, cb, 100);
	ofLine(x, y, lp.x, lp.y);
	ofDisableAlphaBlending();
	
	//Draw the small ball
	ofSetColor(cr, cg, cb);
	ofFill();
	ofCircle(x, y, 3);
	ofNoFill();
	ofCircle(x, y, 3);	

	//Draw the big ball
	ofEnableAlphaBlending();
	ofSetColor(cr, cg, cb, 100);
	ofFill();
	ofCircle(x, y, r);
	ofNoFill();
	ofCircle(x, y, r);
	ofDisableAlphaBlending();
	ofDisableSmoothing();
	
	// Draw volume
	ofDrawBitmapString("volume: " + ofToString(vol), x+r+4, y+1);
}

void ball::drawBounce(float _x, float _y, float _alphaValue){
	if(_alphaValue<255){
		ofEnableAlphaBlending();
		ofSetColor(cr, cg, cb, _alphaValue*fadeOutAlpha);
	}else{
		ofSetColor(cr, cg, cb);
	}
	
	ofEnableSmoothing();
	ofSetColor(cr, cg, cb, _alphaValue*fadeOutAlpha);
	
	ofFill();
	ofCircle(_x, _y, r-5);
	ofNoFill();
	ofSetLineWidth(1);
	ofCircle(_x, _y, r-5);		
	ofSetLineWidth(2);
	ofCircle(_x, _y, r+1);		
	ofDisableSmoothing();	
	if(_alphaValue<255){
		ofDisableAlphaBlending();
	}
}

void ball::drawMass(float alphaValue){
}

void ball::drawNeutral(float _x, float _y, float _alphaValue){
	ofEnableSmoothing();
	ofEnableAlphaBlending();
	
	ofSetColor(cr, cg, cb, _alphaValue*fadeOutAlpha);
	
	ofFill();
	ofCircle(_x, _y, r);
	ofNoFill();
	ofCircle(_x, _y, r);
	
	ofSetColor(cr, cg, cb, (_alphaValue*fadeOutAlpha)/2);
	ofFill();
	ofCircle(_x, _y, r/1.5);

	ofDisableAlphaBlending();
	ofDisableSmoothing();	
}

void ball::drawErase(float _x, float _y, float _alphaValue){
	ofEnableAlphaBlending();
	ofSetColor(255, 0, 0, _alphaValue*fadeOutAlpha);
	ofFill();
	ofCircle(_x, _y, r);
	ofEnableSmoothing();
	ofNoFill();
	ofCircle(_x, _y, r);
	ofLine(_x-r*4, _y-r*4, _x+r*4, _y+r*4);			
	ofLine(_x-r*4, _y+r*4, _x+r*4, _y-r*4);			
	ofDisableSmoothing();
	ofDisableAlphaBlending();
}

void ball::drawBuffer(){
	if(ballIsMoving){
		for (int i = 0; i < bufferPts.size(); i++){
			float alphaValue = ofMap(i, 0, bufferPts.size(), 0, 60);
			
			if(erasingMe){
				drawErase(bufferPts[i].x, bufferPts[i].y, alphaValue);		
			}else if(bounce){
				drawBounce(bufferPts[i].x, bufferPts[i].y, alphaValue);
			}else if(mass){
				drawNeutral(bufferPts[i].x, bufferPts[i].y, alphaValue);
			}else{
				drawNeutral(bufferPts[i].x, bufferPts[i].y, alphaValue);
			}
		}
	}
}