#ifndef _BALL_H
#define _BALL_H

#include "ofMain.h"
#include "ofxVectorMath.h"
#include "ofxOsc.h"
#include "area.h"
#include "segment.h"
#include "myUtils.h"
#include "wave.h"
#include "tremor.h"

class ball : public ofxPoint2f{

	public:
		
		ball();
		~ball();
		ball(ofxPoint2f bornPosition, area a);
		ball(ofxPoint2f bornPosition, area _a, float _r, 
			 ofxOscSender _sOsc);
		
		void finishCreation(ofxPoint2f _lp, int _id);
		void checkCreationConditions(vector <ball> _bList, ofxPoint2f _lp);
		
		void update();
		void checkQuadWall();
		void checkShapeWall();
		void checkObjectCollision(ball oBall);
		ball getCollBall();
		
		void sendOscMessage(string _type, string _action="none");
		void killSound();
		
		void setType(int _cr, int _cg, int _cb, int _s);
		void setErasingMe(bool _e);
		bool getErasingMe();
		bool getKillMe();
		bool getDissapeared();
		void setDissapeared(bool _d);
		bool getCollided();
		void setCollided(bool _c);
		void setBounce(bool _bounce);
		bool getBounce();
		void setMass(bool _mass);
		void setFriction(bool _friction);
		void setSoundId(int _soundId);
		bool getIsMoving();
		int getRcolor();
		int getGcolor();
		int getBcolor();
		void addWave(float _x, float _y, float _r, int _cr, int _cg, int _cb);
		
		void draw();
		void drawSetup(ofxPoint2f _lp);

		float		r;		// radius
		ofxVec2f	v;		// velocity
		float		m;		// mass
		bool		soundStoped;
		
	private:
		void setVelocity(ofxPoint2f _lp);
		void setVolume(float _sin, float _cos);
		void setSpeed();
		void setPan();
		void setRadius(ofxPoint2f _lp);
		void setSamplePosition();
		void setSamplePitch();
		void addTremor();
		
		void drawBounce(float _x, float _y, float _alphaValue);
		void drawMass(float _alphaValue);
		void drawNeutral(float _x, float _y, float _alphaValue);
		void drawErase(float _x, float _y, float _alphaValue);
		
		// Friction stuff
		void addPointToBuffer();
		void drawBuffer();
	
		float		f, speed;		// friction, speed
		area myArea;
		bool creationFinished, ballIsMoving, isReallyMoving;
		bool erasingMe, killMe, collided, dissapear;
		bool bounce, mass, friction; //brk
		
		// Color
		int cr, cg, cb;
		int cNr, cNg, cNb;
		int cYr, cYg, cYb;
		
		ofxPoint2f previousPos;

		ball * ballCollList;
		
		//Visual Feedback
		vector <wave>	waveList;
		vector <tremor> tremorList;
		float fadeOutAlpha;
		
		//Friction
		vector <ofPoint> bufferPts;
		int tailSize;

		//Sound
		ofxOscSender	senderOsc;
		int				sample;
		float			vol, pan, position, pitch;
		int				extressTime;
		int				myId, mySoundId;
};

#endif