#include "guiSettingsManager.h"

guiSettingsManager::guiSettingsManager(){
	index = 0;	//current setting
	selected 		= 0;    //which setting is selected?

	for(int i = 0; i < MAX_SETTINGS; i++) bI[i] = false; 
}

// ------------------------
void guiSettingsManager::readFromFile(string filePath){
	xml.loadFile(filePath);
}

// ---- addSetting()  int
// ---- displayName - name to draw on screen
// ---- xmlName ----- name to look at the xml file
// ---- value ------- value of the setting - int
// ---- min --------- minimum possible value
// ---- max --------- maximum possible value
// ---- incr -------- how much increments the value when poeple interact
void guiSettingsManager::addSetting(string displayName, string xmlName, int value, int min, int max, int incr){
	bI[index] = true; //we know to display it as an int
	addSetting(displayName, xmlName, (float)value, (float)min, (float)max, (float)incr);

}

// ------------------------------- addSetting()  float
// ------------------------------- displayName - name to draw on screen
// ------------------------------- xmlName ----- name to look at the xml file
// ------------------------------- value ------- value of the setting - float
// ------------------------------- min --------- minimum possible value
// ------------------------------- max --------- maximum possible value
// ------------------------------- incr -------- how much increments the value when poeple interact
void guiSettingsManager::addSetting(string displayName, string xmlName, float value, float min, float max, float incr){
	if(index >= MAX_SETTINGS)return;

	gs[index].setName(xmlName, displayName);
	gs[index].setValue(value);
	gs[index].setMinMax(min,max);
	gs[index].setIncr(incr);
//	gs[index].setGroupID(section);
	index++;	
	
	//now we actually read the value from
	//the xml file into our setting
	updateValueFromXml(index-1);
}

// ------------------------
void guiSettingsManager::updateValueFromXml(int which){
	//if(which >= index)return;
	
	if(bI[which]){
		int tmp = gs[which].getValueI();
		tmp = xml.getValue(gs[which].getXmlName(), tmp);
		gs[which].setValue(tmp);	
	}else{
		float tmp = gs[which].getValueF();
		tmp = xml.getValue(gs[which].getXmlName(), tmp);
		gs[which].setValue(tmp);
	}		

	//don't want to trigger changed events from xml
	//gs[which].setChanged(false);
}

// ------------------------------- reloadSettings()
void guiSettingsManager::reloadSettings(){
	for(int i = 0; i < index; i++){
		updateValueFromXml(i);
	}
}

// ------------------------------- saveToFile()
// ------------------------------- filePath -- path to the xml settings file
void guiSettingsManager::saveToFile(string filePath){
	
	for(int i = 0; i < index; i++){
	
		if(bI[i])xml.setValue(gs[i].getXmlName(), gs[i].getValueI());
		else xml.setValue(gs[i].getXmlName(), gs[i].getValueF());
	}
	
	xml.saveFile(filePath);
}

// ------------------------------- getI()
// ------------------------------- varName -- name of the setting
int guiSettingsManager::getI(string varName){
	
	for(int i = 0; i < index; i++){
		if( varName == gs[i].getXmlName() ){
			return gs[i].getValueI();
		}
	}
	return 0;
}	

// ------------------------------- getF()
// ------------------------------- varName -- name of the setting
float guiSettingsManager::getF(string varName){
	for(int i = 0; i < index; i++){
		if( varName == gs[i].getXmlName() ){
			return gs[i].getValueF();
		}
	}
	return 0;
}

// ------------------------------- setI()
// ------------------------------- varName -- name of the setting
// ------------------------------- val ------ int value of the setting
void guiSettingsManager::setI(string varName, int val){
	for(int i = 0; i < index; i++){
		if( varName == gs[i].getXmlName() ){
			gs[i].setValue(val);
		}
	}
}	

// ------------------------------- setF()
// ------------------------------- varName -- name of the setting
// ------------------------------- val ------ float value of the setting 
void guiSettingsManager::setF(string varName, float val){
	for(int i = 0; i < index; i++){
		if( varName == gs[i].getXmlName() ){
			return gs[i].setValue(val);
		}
	}
}


// ------------------------------- selectNext()
// ------------------------------- varName -- name of the setting
bool guiSettingsManager::hasChanged(string varName){
	for(int i = 0; i < index; i++){
		if( varName == gs[i].getXmlName() ){
			return gs[i].getChanged();
		}
	}
	return false;
}

// ------------------------------- selectNext()
void guiSettingsManager::selectNext(){
	selected++;
	if(selected >= index)selected = selected-1;
	
//	whichSection = gs[selected].getGroupID();
}

// ------------------------------- selectPrev()
void guiSettingsManager::selectPrev(){
	selected--;
	if(selected < 0)selected = 0;
	
//	whichSection = gs[selected].getGroupID();
}

// ------------------------------- increase()
void guiSettingsManager::increase(){
	gs[selected].increase();
}

// ------------------------------- decrease()
void guiSettingsManager::decrease(){
	gs[selected].decrease();
}

// ------------------------------- drawAll()
// ------------------------------- x 	
// ------------------------------- y 
// ------------------------------- spacing	
// ------------------------------- lineHeight
void guiSettingsManager::drawAll(float x, float y, float spacing, float lineHeight){
	
	int pos = 0;	
				
	for(int i = 0; i < index; i++){
		
		float drawPosY = y + ( (float)pos*lineHeight);
				
		//this is where we draw the setting names
		//and their values
		i != selected ? glColor4f(0.6f, 0.6f, 0.6f, 1.0f) : glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		
		// Draw the name of the present
		drawText( gs[i].getDisplayName(), x, drawPosY);
	
		// Draw the value of the present
		if(bI[i]){
			handyString = ofToString(gs[i].getValueI());
		}else{  
			handyString = ofToString(gs[i].getValueF());
		}
		
		// Draw square brackets for the selected item.
		if(i == selected) {
			handyString = "["+handyString+"]";
			drawText( handyString, x + spacing, drawPosY);
		}else{
			drawText( handyString, x + spacing-1, drawPosY);
		}
		
		// Go to the next setting
		pos++;
	}
}
