#include "imageProjection.h"

imageProjection::imageProjection(){
	creatingArea = false;
	creatingBall = false;
	deletingBalls = false;
	areaListVisible = false;
	letCreateArea = false;
}

void imageProjection::setup(int _w, int _h, int _x, int _y, 
							string _host, int _sPort, int _rPort){
	width = _w;
	height = _h;
	xoo = _x;
	oyo = _y;
	
	//OSC setup
	sender.setup(_host, _sPort);
	receiver.setup(_rPort);
	
	// GUI 2
	guiArea = ofGetScreenHeight()-100;

	// GUI Radios setup
	int marginBottom = ofGetScreenHeight()-35;
	int marginLeft = 150;
	int marginTwoLeft = 430;
	int marginX = 45;
	int marginTwoX = 140;
	numOfRadios = 5;
	guiRadioList = new guiRadio [numOfRadios];
	guiRadioList[0] = guiRadio(xoo + marginLeft,				marginBottom, 255,  47, 199, 3, 0, guiRadioList, numOfRadios, "sounds", false);
	guiRadioList[1] = guiRadio(xoo + marginLeft + marginX*1,	marginBottom,	70, 179, 255, 3, 1, guiRadioList, numOfRadios, "", false);
	guiRadioList[2] = guiRadio(xoo + marginLeft + marginX*2,	marginBottom, 56,   217, 0, 3, 2, guiRadioList, numOfRadios, "", false);
	guiRadioList[3] = guiRadio(xoo + marginLeft + marginX*3,	marginBottom, 255,  198,   0, 3, 3, guiRadioList, numOfRadios, "", false);
	guiRadioList[4] = guiRadio(xoo + marginLeft + marginX*4+30, marginBottom, 225,   0,   0, 3, 4, guiRadioList, numOfRadios, "delete them", true);
	guiRadioList[3].setChecked(true);

	bounceCheck = guiCheck(xoo + marginTwoLeft,				   marginBottom, 151, 103, 137, 5, "collision");
	massCheck = guiCheck(  xoo + marginTwoLeft + marginTwoX*1, marginBottom, 151, 103, 137, 5, "mass");
	breakCheck = guiCheck( xoo + marginTwoLeft + marginTwoX*2, marginBottom, 151, 103, 137, 5, "friction");


	// GUI
//	guiArea = xoo + 200;
	
	// GUI Radios setup
/*	
	int marginTop = 80;
	int marginTwoTop = 22;
	int marginLeft = 100;
	int marginY = 40;
	int marginTwoY = 58;
	numOfRadios = 5;
	guiRadioList = new guiRadio [numOfRadios];
	guiRadioList[0] = guiRadio(xoo + marginLeft, oyo + marginTop,			     255,   3, 184, 3, 0, guiRadioList, numOfRadios, "sound 1", false);
	guiRadioList[1] = guiRadio(xoo + marginLeft, oyo + marginTop + marginY*1,      0, 100, 255, 3, 1, guiRadioList, numOfRadios, "sound 2", false);
	guiRadioList[2] = guiRadio(xoo + marginLeft, oyo + marginTop + marginY*2,    156,   0, 255, 3, 2, guiRadioList, numOfRadios, "sound 3", false);
	guiRadioList[3] = guiRadio(xoo + marginLeft, oyo + marginTop + marginY*3,    255,  94,   0, 3, 3, guiRadioList, numOfRadios, "sound 4", false);
	guiRadioList[4] = guiRadio(xoo + marginLeft, oyo + marginTop + marginY*4+30, 225,   0,   0, 3, 4, guiRadioList, numOfRadios, "delete balls", true);
	guiRadioList[3].setChecked(true);

	bounceCheck = guiCheck(xoo + marginLeft, oyo + marginTop + marginTwoY*4 + marginTwoTop, 151, 103, 137, 5, "collision");
	massCheck = guiCheck(  xoo + marginLeft, oyo + marginTop + marginTwoY*5 + marginTwoTop, 151, 103, 137, 5, "mass");
	breakCheck = guiCheck( xoo + marginLeft, oyo + marginTop + marginTwoY*6 + marginTwoTop, 151, 103, 137, 5, "friction");
*/	
	// XML Area
	loadAreaList();
}

void imageProjection::update(){
	// Create Area
	createArea();
	
	// Create Balls
	createBalls();
	
	// update balls
	for(int i=0; i<ballList.size(); i++){
		
		// normal update
		ballList[i].update();
		
		// update while creating ball
		ballList[ballList.size()-1].checkCreationConditions(ballList, laserPosition);
		
	}	
	
	if(ballList.size()>1){
	
		// If they are ** BOUNCE ** balls
		for(int i=0; i<ballList.size(); i++){
			if(ballList[i].getBounce()){
				
				for(int j=i+1; j<ballList.size(); j++){
				
					if( ballList[j].getBounce() && (ballList[i].getIsMoving() || ballList[j].getIsMoving()) ){
						ballList[i].checkObjectCollision(ballList[j]);
						ballList[j] = ballList[i].getCollBall();
					
						if(ballList[i].getKillMe() && ballList[i].getDissapeared()){
							ballList.erase(ballList.begin()+i);
						}			
					}
				}
			}
		}		
	}
	
	// Check GUI
	if(checkLaser){
		for(int i=0; i<numOfRadios; i++){
			guiRadioList[i].press(laserPosition);		
		}
	}
	bounceCheck.press(laserPosition, checkLaser);
	massCheck.press(laserPosition, checkLaser);
	breakCheck.press(laserPosition, checkLaser);
	
	// Check Delete Balls
	deleteBalls();
	
	// Receive OSC
	receiveOsc();
}

void imageProjection::draw(){
	// draw laser pointer
	drawLaserPointer();
	
	  // draw line cration area
	if(creatingArea) areaList[areaOnCreation].drawCreationLine(laserPosition);
	
	// draw areas
	if(areaListVisible){ 
		for(int i=0; i<areaList.size(); i++){
			areaList[i].draw();
		}		
	}
	
	// draw balls
	for(int i=0; i<ballList.size(); i++){
		if(ballList.size()-1 == i){
			if(creatingBall){
				ballList[i].drawSetup(laserPosition);
			}else{
				ballList[i].draw();
			}
		}else{
			ballList[i].draw();
		}
	}		
	
	// Draw GUI
	for(int i=0; i<numOfRadios; i++){
		guiRadioList[i].draw();		
	}
	bounceCheck.draw();
	massCheck.draw();
	breakCheck.draw();
	
	// Draw delete selection
	if(deletingBalls){
		delSelection[0].draw();
	}
}

void imageProjection::drawLaserPointer(){
	if(checkLaser){
		ofEnableSmoothing();
		ofSetColor(255, 0, 100);
		ofNoFill();
		ofCircle(laserPosition.x, laserPosition.y, 15);
		ofDisableSmoothing();
	}
}

void imageProjection::relocateLaserPos(ofxPoint2f _lp){
	laserPosition.x = xoo + _lp.x;
	laserPosition.y = oyo + _lp.y;
}

void imageProjection::setLaserStatus(bool _lStat){
	checkLaser = _lStat;
}

void imageProjection::setAreaResolution(float _v){
	areaResolution = _v;
}

void imageProjection::setCreateArea(bool _a){
	letCreateArea = _a;
}

void imageProjection::setAreasVisibility(bool _a){
	areaListVisible = _a;
}

void imageProjection::deleteAreas(){
	if(areaList.size()>0){
		areaList.clear();
	}
}

//--------------------------------------------------------------
void imageProjection::keyPressed(int key){

	// Move area
	if(key=='m'){
		for(int i=0; i<areaList.size(); i++){
			areaList[i].moveArea(laserPosition);
		}				
	}
	
	//Delete area
	if(key=='d'){
		for(int i=0; i<areaList.size(); i++){
			if(areaList[i].isLaserOver(laserPosition)){
				areaList[i].deleteArea();
				areaList.erase(areaList.begin()+i);
			}
		}						
	}
	
	if(key=='x'){
		saveAreaList();
	}
}

//--------------------------------------------------------------
void imageProjection::keyReleased(int key){
}

void imageProjection::mouseDragged(int x, int y, int button){
//	if(x > xoo + 200){
		for(int i=0; i<areaList.size(); i++){
			areaList[i].mouseDragged(x, y, button);
		}
//	}
}

void imageProjection::mousePressed(int x, int y, int button){
//	if(x > xoo + 200){
		for(int i=0; i<areaList.size(); i++){
			areaList[i].mousePressed(x, y, button);
		}
//	}
}

void imageProjection::mouseReleased(int x, int y, int button){
//	if(x > xoo + 200){
		for(int i=0; i<areaList.size(); i++){
			areaList[i].mouseReleased(x, y, button);
		}
//	}
}

//-----------------------------------------------------------------
void imageProjection::createArea(){
	if(letCreateArea){
		if(checkLaser){
			if(creatingArea){
				areaList[areaOnCreation].build(laserPosition);
			}else{ 
				areaList.push_back(area());	
				areaOnCreation = areaList.size()-1;
				areaList[areaOnCreation].create(laserPosition, areaResolution);

				creatingArea = true;
			}
		}else{
			if(creatingArea){
				areaList[areaOnCreation].finishBuilding();
			}
			creatingArea = false;
		}
	}
}

void imageProjection::createBalls(){
	
	// Check if delete balls button is pressed.
	bool itIsEraser = false;
	for(int j=0; j<numOfRadios; j++){
		if(guiRadioList[j].getChecked() && guiRadioList[j].getEraser()){
			itIsEraser = true;
		}
	}
	
	// Create balls
	if(!creatingArea && !itIsEraser){
		if(checkLaser){
			for(int i=0; i<areaList.size(); i++){
				if(areaList[i].isLaserOver(laserPosition) && !creatingBall){
					ballList.push_back(ball(laserPosition, areaList[i], 8, sender));
					
					// Bounce ball
					if(bounceCheck.getChecked()){
						ballList[ballList.size()-1].setBounce(true);
					}else{
						ballList[ballList.size()-1].setBounce(false);
					}

					// Mass ball
					if(massCheck.getChecked()){
						ballList[ballList.size()-1].setMass(true);
					}else{
						ballList[ballList.size()-1].setMass(false);
					}

					// Friction ball
					if(breakCheck.getChecked()){
						ballList[ballList.size()-1].setFriction(true);
					}else{
						ballList[ballList.size()-1].setFriction(false);
					}
					
					// get the type of ball from the radio GUI list
					// type of ball means choose color and sound ball
					for(int j=0; j<numOfRadios; j++){
						if(guiRadioList[j].getChecked()){
							int * color = guiRadioList[j].getColor();
							ballList[ballList.size()-1].setType(color[0], color[1], color[2], j);
						}		
					}
					creatingBall = true;
				}
			}			
		}else if(creatingBall){
			if(ballList[ballList.size()-1].getKillMe()){
				ballList.pop_back();
			}else{
				int lastBallIdx = ballList.size()-1;
				ballList[lastBallIdx].finishCreation(laserPosition, lastBallIdx);
			}
			creatingBall = false;
		}
	}
}

void imageProjection::deleteBalls(){
	if(laserPosition.x > guiArea){
		if(checkLaser){
			// Check if the erase button is pressed
			bool amIeraser = false;
			for(int i=0; i<numOfRadios; i++){
				if(guiRadioList[i].getChecked() && guiRadioList[i].getEraser()){
					amIeraser = true;
				}	
			}		
		
			if(amIeraser){
				if(!deletingBalls){
					// set the initial coordinates (top-left corner)
					// of the selection rectangle
					delSelection.push_back(	selection()	);
					// set initial position of deleting selection
					delSelection[0].set(laserPosition.x, laserPosition.y);
					deletingBalls = true;
				}else{
					// update min and max points of deleting selection
					delSelection[0].update(laserPosition.x, laserPosition.y);
					// check if the ball is inside the selection
					for(int i=0; i<ballList.size(); i++){
						if(	ballList[i].x > delSelection[0].getMinX() &&
						ballList[i].x < delSelection[0].getMaxX() &&
						ballList[i].y > delSelection[0].getMinY() &&
						ballList[i].y < delSelection[0].getMaxY() ){
						
							// if the ball is inside, we tell the ball
							// that he is going to be deleted
							ballList[i].setErasingMe(true);
						}else if(ballList[i].getErasingMe()){
							// if the ball is outside the selection but
							// it was previosly set to is going to be
							// deleted, we tell it that it is not going
							// to be deleted
							ballList[i].setErasingMe(false);
						}
					}			
				}
			}
		}else if(deletingBalls){		
			deletingBalls = false;
		}else{
			// Here we erase the balls from the array
			for(int i=0; i<ballList.size(); i++){
				if(ballList[i].getErasingMe()){
					ballList[i].killSound();
					ballList.erase(ballList.begin()+i);
				}
			}
			delSelection.clear();			
		}
	}
}

void imageProjection::deleteAllBalls(){
	ballList.clear();		
}

void imageProjection::loadAreaList(){
	xmlAreas.loadFile("settings_areas.xml");
	
	int numAreas = xmlAreas.getNumTags("AREA");
	if(numAreas > 0){
		for(int i=0; i<numAreas; i++){
			// Create a new area
			areaList.push_back(area());

			xmlAreas.pushTag("AREA", i);
			int numVertex = xmlAreas.getNumTags("PT");
			
			// Create vertex of the area
			if(numVertex > 0){
				for(int j=0; j<numVertex; j++){
					int xPt = xmlAreas.getValue("PT:X", 0, j);
					int yPt = xmlAreas.getValue("PT:Y", 0, j);
					ofxPoint2f vertex;
					vertex.set(xPt, yPt);
					
					areaList[i].setAreaVertex(vertex);
				}					
			}
			areaList[i].finishBuilding();
			xmlAreas.popTag();
		}
	}
}

void imageProjection::saveAreaList(){
	xmlAreas.clear();
// <AREA_LIST>
	for(int i=0; i<areaList.size(); i++){
//	<AREA>		
		int vertexListSize = areaList[i].getNumAreaVertex();
		int a = xmlAreas.addTag("AREA");
		xmlAreas.pushTag("AREA", a);

		for(int j=0; j<vertexListSize; j++){
//		<PT>
			ofxPoint2f p = areaList[i].getAreaVertex(j);
			int b = xmlAreas.addTag("PT");
//			<X>
			xmlAreas.setValue("PT:X", p.x, b);
//			</X>
//			<Y>
			xmlAreas.setValue("PT:Y", p.y, b);
//			</Y>
//		</PT>
		}
		xmlAreas.popTag();
//	</AREA>				
	}
// </AREA_LIST>				

	xmlAreas.saveFile("settings_areas.xml");	
}

void imageProjection::receiveOsc(){
	while(receiver.hasWaitingMessages()){
		ofxOscMessage m;
		receiver.getNextMessage(&m);
		if(m.getAddress()=="/soundId"){
			int ballId = m.getArgAsInt32(0);
			int soundId = m.getArgAsInt32(1);
			ballList[ballId].setSoundId(soundId);							
		}
	}
}
