#ifndef _IMAGE_PROJECTION_H
#define _IMAGE_PROJECTION_H

#include "ofMain.h"
#include "ofxVectorMath.h"
#include "ofxOsc.h"
#include "ofxXmlSettings.h"
#include "baseGui.h"
#include "area.h"
#include "ball.h"
#include "guiRadio.h"
#include "guiCheck.h"
#include "selection.h"

class imageProjection : public baseGui{

	public:
	
		imageProjection();
		
		void setup(int _w, int _h, int _x, int _y, 
				   string _host, int _sPort, int _rPort);
		void update();
		
		void draw();		
		void drawLaserPointer();
		
		void relocateLaserPos(ofxPoint2f _lp);
		void setLaserStatus(bool _lCheck);
		void setAreaResolution(float _v);
		void setCreateArea(bool _a);
		void setAreasVisibility(bool _a);
		void deleteAreas();
		void deleteAllBalls();

		void keyPressed(int key);
		void keyReleased(int key);

		void mouseDragged(int x, int y, int button);
		void mousePressed(int x, int y, int button);
		void mouseReleased(int x, int y, int button);

	private:
	
		void createArea();
		void createBalls();
		void deleteBalls();
		
		void loadAreaList();
		void saveAreaList();
		
		void receiveOsc();
		
		int width, height;
		int xoo, oyo;
		bool creatingArea, creatingBall, areaListVisible, 
			 aKeyPressed, deletingBalls, letCreateArea;
		int areaOnCreation;
		
		ofxPoint2f  laserPosition;
		bool		checkLaser;
	
		vector <area> areaList;
		vector <ball> ballList;
		
		guiRadio *	guiRadioList;
		int			numOfRadios;
		int			guiArea;
		guiCheck	bounceCheck;
		guiCheck	massCheck;
		guiCheck	breakCheck;

		ofxOscSender	sender;	
		ofxOscReceiver	receiver;
		
		//XML
		ofxXmlSettings xmlAreas;
		
		vector <selection> delSelection;
		
		float areaResolution;
};

#endif