#include "laserTracking.h"

laserTracking::laserTracking(){
}

void laserTracking::setup(int _wi, int _hi, int _wo, int _ho){

	widthInput = _wi;
	heightInput = _hi;
	widthOutput = _wo;
	heightOutput = _ho;

	dilateCheckLaser = false;
	countDilateLaser = 0;
	countDilateLimit = 40;
	
	xVidPos = 0;
	yVidPos = 0;
	// Video grabber
	vidGrabber.setVerbose(true);
	vidGrabber.initGrabber(widthInput, heightInput);

	// Allocations
	colorImg.allocate(widthInput, heightInput);		// Live Cam as input source
	grayImg.allocate(widthInput, heightInput);
	warpedTrackedTexture.allocate(widthInput, heightInput);	// Gray Live Cam to warped
	warpedGrayImg.allocate(widthInput, heightInput);
    colorImgHSV.allocate(widthInput, heightInput);	// HSV Image
    hueImg.allocate(widthInput, heightInput);		// Hue Image
    satImg.allocate(widthInput, heightInput);		// Saturation Image
    valImg.allocate(widthInput, heightInput);		// value Image

    colorTrackedPixels = new unsigned char [widthInput * heightInput];	// Tracked Image
    trackedTexture.allocate(widthInput, heightInput);
}

void laserTracking::setupWarpIn(ofPoint _srcPos0, ofPoint _srcPos1, ofPoint _srcPos2, 
								ofPoint _srcPos3, ofPoint _topLeftPos){
    guiIn = ofRectangle(_topLeftPos.x, _topLeftPos.y, widthInput, heightInput);

	srcPositions  = new ofPoint[4];
	srcPositions[0] = _srcPos0;
	srcPositions[1] = _srcPos1;
	srcPositions[2] = _srcPos2;
	srcPositions[3] = _srcPos3;

	dstPositions  = new ofPoint[4];
	dstPositions[0].set(0, 0);
	dstPositions[1].set(widthInput, 0);
	dstPositions[2].set(widthInput, heightInput);
	dstPositions[3].set(0, heightInput);	
	
	topLeftPos = _topLeftPos;
}

void laserTracking::processFrame(float hue, float hueWidth, float sat, float satWidth, float val, 
								 float valWidth, int blobSize){
	vidGrabber.grabFrame();


//	if(vidGrabber.isFrameNew()){
		//Calculate min and max thersholds values
		float minHue = max((hue - hueWidth*0.5) * 255, 0.0);
		float maxHue = min((hue + hueWidth*0.5) * 255, 255.0);
		float minSat = max((sat - satWidth*0.5) * 255, 0.0);
		float maxSat = min((sat + satWidth*0.5) * 255, 255.0);
		float minVal = max((val - valWidth*0.5) * 255, 0.0);
		float maxVal = min((val + valWidth*0.5) * 255, 255.0);
		
		colorImg.setFromPixels(vidGrabber.getPixels(), widthInput, heightInput);	
		grayImg.setFromColorImage(colorImg);
		warpedGrayImg.warpIntoMe(grayImg, srcPositions, dstPositions);
			
		// HSV
		colorImgHSV = colorImg;
		colorImgHSV.convertRgbToHsv();
		colorImgHSV.convertToGrayscalePlanarImages(hueImg, satImg, valImg);
	
		// Perform tracking calculations
		unsigned char * huePixels = hueImg.getPixels();
		unsigned char * satPixels = satImg.getPixels();
		unsigned char * valPixels = valImg.getPixels();
		int nPixels = widthInput * heightInput;
	
		for (int i = 0; i < nPixels; i++){
			if ((huePixels[i] >= minHue && huePixels[i] <= maxHue) &&
				(satPixels[i] >= minSat && satPixels[i] <= maxSat) &&
				(valPixels[i] >= minVal && valPixels[i] <= maxVal)){
				colorTrackedPixels[i] = 255;
			} else {
				colorTrackedPixels[i] = 0;
			}
		}
	
		trackedTexture.setFromPixels(colorTrackedPixels, widthInput, heightInput);
		// dilate white pixels, it will make the blob bigger.
		trackedTexture.dilate();
		trackedTexture.dilate();
		warpedTrackedTexture.warpIntoMe(trackedTexture, srcPositions, dstPositions);
	
		// ******Blobs
		// find contours which are between the size of 20 pixels and 1/3 the w*h pixels.
		// also, find holes is set to true so we will get interior contours as well....
		contourFinder.findContours(warpedTrackedTexture, blobSize, (widthInput*heightInput)/3, 30, false);	// find holes	

//	}
}

void laserTracking::calcColorRange(float hue, float hueThresh, float sat, float satThresh, float value){
	
	//ehh this is kind of ugly and needs to be fixed I think
	//Lets calc the rgb values for the min and max hue range			
	float rgb[3];
	float hsb[3];
	
	hsb[0] = hue-hueThresh*0.5;
	hsb[1] = sat-satThresh*0.5;
	hsb[2] = value;
	
	hsbToRgb(hsb, rgb);
	
	r0Min = (int)(rgb[0]*255);
	g0Min = (int)(rgb[1]*255);
	b0Min = (int)(rgb[2]*255);
	
	hsb[0] = hue + hueThresh * 0.5;
	hsb[1] = sat + satThresh * 0.5;
	hsb[2] = value;
			
	hsbToRgb(hsb, rgb);
						
	r0Max = (int)(rgb[0]*255);
	g0Max = (int)(rgb[1]*255);
	b0Max = (int)(rgb[2]*255);
	
	hsb[0] = hue-hueThresh*0.5;
	if(sat == 0) hsb[1] = 0.0;
	else hsb[1] = 1.0;			
	hsb[2] = 1.0;
	
	hsbToRgb(hsb, rgb);
	
	r1Min = (int)(rgb[0]*255);
	g1Min = (int)(rgb[1]*255);
	b1Min = (int)(rgb[2]*255);
	
	hsb[0] = hue+hueThresh*0.5;
	if(sat == 0) hsb[1] = 0.0;
	else hsb[1] = 1.0;	
	hsb[2] = 1.0;
			
	hsbToRgb(hsb, rgb);
						
	r1Max = (int)(rgb[0]*255);
	g1Max = (int)(rgb[1]*255);
	b1Max = (int)(rgb[2]*255);
}

void laserTracking::drawInput(int _x, int _y){
	ofSetColor(255, 255, 255);
	colorImg.draw(_x, _y);	
}

void laserTracking::drawWarpIn(){

	// Draw lines between points
	ofSetColor(0x33DD44);
	ofNoFill();
	ofBeginShape();
	for (int j = 0; j < 4; j++){
		ofVertex(srcPositions[j].x + topLeftPos.x, srcPositions[j].y + topLeftPos.y);
	}
	ofEndShape(true);

	// Draw points
	ofFill();
	ofSetColor(0x3344FF);
	for (int j = 0; j < 4; j++){
		ofCircle(srcPositions[j].x + topLeftPos.x, srcPositions[j].y + topLeftPos.y, 3);
	}
}

void laserTracking::drawTrack(int _x, int _y){
	ofSetColor(255, 255, 255);
	warpedTrackedTexture.draw(_x, _y);

	// ******Blobs
	// we could draw the whole contour finder
	//contourFinder.draw(360,540);
	// or, instead we can draw each blob individually,
	// this is how to get access to them:
    for (int i = 0; i < contourFinder.nBlobs; i++){
        contourFinder.blobs[i].draw(_x, _y);
    }

	ofSetColor(50, 50, 50);
	ofNoFill();
	ofRect(_x, _y, widthInput, heightInput);
	ofFill();
	
}

void laserTracking::drawWarpGray(int _x, int _y){
	ofSetColor(255, 255, 255);
	warpedGrayImg.draw(_x, _y);	
	ofSetColor(50, 50, 50);
	ofNoFill();
	ofRect(_x, _y, widthInput, heightInput);
	ofFill();
}

// ------------------------------- drawColorRange()
// ------------------------------- x -- x position 	
// ------------------------------- y -- y position
// ------------------------------- w -- width 	
// ------------------------------- h -- height
void laserTracking::drawColorRange(float x, float y, float w, float h){

	//this is not super accurate
	//but good for an indication
	glBegin(GL_QUADS);
		ofSetColor(r1Min, g1Min, b1Min);
		glVertex2f(x,  y);
		
		ofSetColor(r1Max, g1Max, b1Max);				
		glVertex2f(x+w, y);
		
		ofSetColor(r0Max, g0Max, b0Max);
		glVertex2f(x+w, y + h );

		ofSetColor(r0Min, g0Min, b0Min);
		glVertex2f(x, y + h);
	glEnd();
}

void laserTracking::videoConfig(){
	vidGrabber.videoSettings();
}

bool laserTracking::checkLaser(){
	if(contourFinder.nBlobs > 0){
		dilateCheckLaser = true;
		countDilateLaser = 0;			
		return true;
	}else{
		if(dilateCheckLaser && countDilateLaser < countDilateLimit){
			countDilateLaser++;
			return true;
		}else{
			dilateCheckLaser = false;
			countDilateLaser = 0;			
			return false;
		}
	}
}

void laserTracking::mouseDragged(int x, int y, int button){
    if (activePointIn > -1){
        srcPositions[activePointIn].x = x - topLeftPos.x;
        srcPositions[activePointIn].y = y - topLeftPos.y;
	}
}

void laserTracking::mousePressed(int x, int y, int button){
    //this is not the best way
    activePointIn       = -1;

    float smallestDist  = 999999;
	float clickRadius = 10;

	for (int j = 0; j < 4; j++){
		ofPoint inputPt;
		inputPt.x = srcPositions[j].x + topLeftPos.x;
		inputPt.y = srcPositions[j].y + topLeftPos.y;
		inputPt.z = 0;
		float len = sqrt( (inputPt.x - x) * (inputPt.x - x) +
							(inputPt.y - y) * (inputPt.y - y));
		if (len < clickRadius && len < smallestDist){
			activePointIn  = j;
			smallestDist = len;
		}
	}
}

ofxPoint2f laserTracking::getLaserPos(){
	ofxPoint2f tempPos;
	float originalLaserX = contourFinder.blobs[0].centroid.x;
	float originalLaserY = contourFinder.blobs[0].centroid.y;
	tempPos.x = ofMap(originalLaserX, 0.0f, (float)widthInput, 0.0f, (float)widthOutput);
	tempPos.y = ofMap(originalLaserY, 0.0f, (float)heightInput, 0.0f, (float)heightOutput);
			
	return tempPos;
}

ofPoint * laserTracking::getWarpPoints(){
	return srcPositions;
}

void laserTracking::setFreezedLaserTime(float _t){
	countDilateLimit = _t;
}
