#ifndef _LASER_TRACKING_H
#define _LASER_TRACKING_H

#include "ofxOpenCv.h"
#include "ofxVectorMath.h"
#include "laserUtils.h"
#include "baseGui.h"

class laserTracking : public baseGui{

	public:
	
		laserTracking();
		void setup(int wi, int hi, int wo, int ho);
		void setupWarpIn(ofPoint _srcPos0, ofPoint _srcPos1, ofPoint _srcPos2, 
						 ofPoint _srcPos3, ofPoint _topLeftPos);

		void processFrame(float hue, float hueWidth, float sat, float satWidth, float val, 
						  float valWidth, int blobSize);
		void calcColorRange(float hue, float hueThresh, float sat, float satThresh, float value);
		void drawColorRange(float x, float y, float w, float h);
		void draw(float x, float y);
		void drawInput(int _x, int _y);
		void drawWarpIn();
		void drawTrack(int _x, int _y);
		void drawWarpGray(int _x, int _y);
		void videoConfig();

		void mouseDragged(int x, int y, int button);
		void mousePressed(int x, int y, int button);
				
		bool checkLaser();
		ofxPoint2f getLaserPos();
		ofPoint * getWarpPoints();
		void setFreezedLaserTime(float _t);
	
	private:
	
		int r0Min, r0Max, g0Min, g0Max, b0Min, b0Max, r1Min, r1Max, g1Min, g1Max, b1Min, b1Max;
		
		ofVideoGrabber			vidGrabber;

        ofxCvColorImage		    colorImg;
        ofxCvColorImage		    colorImgHSV;
        ofxCvGrayscaleImage		hueImg;
        ofxCvGrayscaleImage		satImg;
        ofxCvGrayscaleImage		valImg;
        ofxCvGrayscaleImage		trackedTexture;
        ofxCvGrayscaleImage		warpedTrackedTexture;
        ofxCvGrayscaleImage		warpedGrayImg;
        ofxCvGrayscaleImage		grayImg;
		ofxCvContourFinder		contourFinder;		

        unsigned char *         colorTrackedPixels;

        ofRectangle				guiIn;
		int widthInput, heightInput, widthOutput, heightOutput;
		int						xVidPos, yVidPos;
        int						activePointIn;

		ofPoint *				srcPositions;
        ofPoint *				dstPositions;
		ofPoint					topLeftPos;
		
		bool					dilateCheckLaser;
		int						countDilateLaser, countDilateLimit;
};

#endif